!******************************************************************************!
!                                                                              !
!    class_vertices.f90                                                        !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2020 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_vertices.f90                             |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2015                                 |            !
!        |       | UFO version: 0.1                               |            !
!        |       | Generation date: 2020-08-03 19:59             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_vertices
  use class_particles, only: n_orders
  use constants_mdl
  use dictionary, only: dict_init
  implicit none
  save

  type Branch
    integer :: lorentz_id
    integer :: perm
    integer, allocatable :: couplings(:)
    integer, dimension(n_orders) :: coupling_order
    integer :: color_id
    integer :: colorfac_id
    ! Propagator extension (only supported for massive vectors)
    ! 0: Usual propagator
    ! 1: Rxi-propagator part with pole @ M_V^2
    ! 2: Rxi-propagator part with pole @ \xi*M_V^2
    integer :: pext
  end type

  type Vertex
    type(Branch), allocatable :: branch(:)
    type(Branch), allocatable :: branchCT(:)
    type(Branch), allocatable :: branchR2(:)
  end type

  logical :: vertices_init = .false.
  logical :: vertices_filled = .false.
  integer, parameter :: n_vertices = 669
  type(Vertex) :: vertices(0:n_vertices-1)

  ! Defines the maximum allowed number of legs participating at a vertex
  integer, parameter :: max_leg_multiplicity = 7

  integer, parameter :: FFaS0 = 1
  integer, parameter :: VSFSFa0 = 2
  integer, parameter :: VVSSVVS0 = 3
  integer, parameter :: VVSSVVS1 = 4
  integer, parameter :: VVSSVVS2 = 5
  integer, parameter :: SVVVS0 = 6
  integer, parameter :: SVVVS1 = 7
  integer, parameter :: FFaV0 = 8
  integer, parameter :: SVVVV0 = 9
  integer, parameter :: FaSFSV0 = 10
  integer, parameter :: SVSVSVV0 = 11
  integer, parameter :: SVSVSVV1 = 12
  integer, parameter :: SVSVSVV2 = 13
  integer, parameter :: FSVSFa0 = 14
  integer, parameter :: VVSVSS0 = 15
  integer, parameter :: FFaVS0 = 16
  integer, parameter :: SVVVSVS0 = 17
  integer, parameter :: SVVVSVS1 = 18
  integer, parameter :: SVVVSVS2 = 19
  integer, parameter :: VVSS0 = 20
  integer, parameter :: VVSS1 = 21
  integer, parameter :: FaSVF0 = 22
  integer, parameter :: SSVVS0 = 23
  integer, parameter :: VVSV0 = 24
  integer, parameter :: VVSV1 = 25
  integer, parameter :: SVVSVVS0 = 26
  integer, parameter :: SVVSVVS1 = 27
  integer, parameter :: SVVSVVS2 = 28
  integer, parameter :: VVV0 = 29
  integer, parameter :: SVVSSVV0 = 30
  integer, parameter :: SVVSSVV1 = 31
  integer, parameter :: SVVSSVV2 = 32
  integer, parameter :: VVS0 = 33
  integer, parameter :: VVS1 = 34
  integer, parameter :: VSVVSVS0 = 35
  integer, parameter :: VSVVSVS1 = 36
  integer, parameter :: VSVVSVS2 = 37
  integer, parameter :: VSFFa0 = 38
  integer, parameter :: FFa0 = 39
  integer, parameter :: FaFVSS0 = 40
  integer, parameter :: FaFSV0 = 41
  integer, parameter :: FFaSV0 = 42
  integer, parameter :: FaFSS0 = 43
  integer, parameter :: VUaU0 = 44
  integer, parameter :: VFSFa0 = 45
  integer, parameter :: FSFaVS0 = 46
  integer, parameter :: SVSSVVV0 = 47
  integer, parameter :: SVSSVVV1 = 48
  integer, parameter :: SVSSVVV2 = 49
  integer, parameter :: SVSVVS0 = 50
  integer, parameter :: SSFaFV0 = 51
  integer, parameter :: FaFS0 = 52
  integer, parameter :: FSVFaS0 = 53
  integer, parameter :: VSVSS0 = 54
  integer, parameter :: VSVSVSV0 = 55
  integer, parameter :: VSVSVSV1 = 56
  integer, parameter :: VSVSVSV2 = 57
  integer, parameter :: FaFV0 = 58
  integer, parameter :: SSSS0 = 59
  integer, parameter :: SFSVFa0 = 60
  integer, parameter :: FaSSVF0 = 61
  integer, parameter :: SFFa0 = 62
  integer, parameter :: SFaF0 = 63
  integer, parameter :: SVFSFa0 = 64
  integer, parameter :: SVV0 = 65
  integer, parameter :: SVV1 = 66
  integer, parameter :: FSFaV0 = 67
  integer, parameter :: FSFaS0 = 68
  integer, parameter :: SVSVVSV0 = 69
  integer, parameter :: SVSVVSV1 = 70
  integer, parameter :: SVSVVSV2 = 71
  integer, parameter :: VSSSVVV0 = 72
  integer, parameter :: VSSSVVV1 = 73
  integer, parameter :: VSSSVVV2 = 74
  integer, parameter :: VSFaFS0 = 75
  integer, parameter :: FSSVFa0 = 76
  integer, parameter :: VVSVSVS0 = 77
  integer, parameter :: VVSVSVS1 = 78
  integer, parameter :: VVSVSVS2 = 79
  integer, parameter :: FFaVSS0 = 80
  integer, parameter :: SVFFa0 = 81
  integer, parameter :: VVVSSS0 = 82
  integer, parameter :: VVVSSV0 = 83
  integer, parameter :: VSSFFa0 = 84
  integer, parameter :: FaFSSV0 = 85
  integer, parameter :: FaSFV0 = 86
  integer, parameter :: SVFFaS0 = 87
  integer, parameter :: VSSVVSV0 = 88
  integer, parameter :: VSSVVSV1 = 89
  integer, parameter :: VSSVVSV2 = 90
  integer, parameter :: VSSSVV0 = 91
  integer, parameter :: FSFa0 = 92
  integer, parameter :: SSVVVSV0 = 93
  integer, parameter :: SSVVVSV1 = 94
  integer, parameter :: SSVVVSV2 = 95
  integer, parameter :: SFaFVS0 = 96
  integer, parameter :: VVSVVSS0 = 97
  integer, parameter :: VVSVVSS1 = 98
  integer, parameter :: VVSVVSS2 = 99
  integer, parameter :: VSSVVVS0 = 100
  integer, parameter :: VSSVVVS1 = 101
  integer, parameter :: VSSVVVS2 = 102
  integer, parameter :: VVSSSV0 = 103
  integer, parameter :: FSVFa0 = 104
  integer, parameter :: SSFVFa0 = 105
  integer, parameter :: SVSVVVS0 = 106
  integer, parameter :: SVSVVVS1 = 107
  integer, parameter :: SVSVVVS2 = 108
  integer, parameter :: VFaFS0 = 109
  integer, parameter :: FSSFaV0 = 110
  integer, parameter :: SVVVVS0 = 111
  integer, parameter :: SVVVVSS0 = 112
  integer, parameter :: SVVVVSS1 = 113
  integer, parameter :: SVVVVSS2 = 114
  integer, parameter :: FaSVSF0 = 115
  integer, parameter :: SFFaV0 = 116
  integer, parameter :: SFaVFS0 = 117
  integer, parameter :: VFaSF0 = 118
  integer, parameter :: VVVS0 = 119
  integer, parameter :: VVVS1 = 120
  integer, parameter :: VSFFaS0 = 121
  integer, parameter :: VVVV0 = 122
  integer, parameter :: SFFaSV0 = 123
  integer, parameter :: SVSV0 = 124
  integer, parameter :: SVSV1 = 125
  integer, parameter :: VVSSSVV0 = 126
  integer, parameter :: VVSSSVV1 = 127
  integer, parameter :: VVSSSVV2 = 128
  integer, parameter :: VSVS0 = 129
  integer, parameter :: SFFaS0 = 130
  integer, parameter :: FaVSSF0 = 131
  integer, parameter :: SFSFa0 = 132
  integer, parameter :: VFaSFS0 = 133
  integer, parameter :: SFaVSF0 = 134
  integer, parameter :: SVFaFS0 = 135
  integer, parameter :: SVSSV0 = 136
  integer, parameter :: VSSVVS0 = 137
  integer, parameter :: VSSSV0 = 138
  integer, parameter :: SSFFaV0 = 139
  integer, parameter :: FVSSFa0 = 140
  integer, parameter :: SFaSF0 = 141
  integer, parameter :: SSVVSV0 = 142
  integer, parameter :: FaSF0 = 143
  integer, parameter :: FaSFS0 = 144
  integer, parameter :: SFaFV0 = 145
  integer, parameter :: SSVV0 = 146
  integer, parameter :: VFFaS0 = 147
  integer, parameter :: SVVVSSV0 = 148
  integer, parameter :: SVVVSSV1 = 149
  integer, parameter :: SVVVSSV2 = 150
  integer, parameter :: SSS0 = 151
  integer, parameter :: SSVSV0 = 152
  integer, parameter :: SFVFa0 = 153
  integer, parameter :: VSVSVS0 = 154
  integer, parameter :: FVSFa0 = 155
  integer, parameter :: VSSFaF0 = 156
  integer, parameter :: VFFa0 = 157
  integer, parameter :: VSVVSS0 = 158
  integer, parameter :: UaUV0 = 159
  integer, parameter :: SSVVVS0 = 160
  integer, parameter :: SVFaF0 = 161
  integer, parameter :: SFaVF0 = 162
  integer, parameter :: VVVSS0 = 163
  integer, parameter :: VVVSS1 = 164
  integer, parameter :: SVVV0 = 165
  integer, parameter :: SVVV1 = 166
  integer, parameter :: VVVSV0 = 167
  integer, parameter :: SSVSVV0 = 168
  integer, parameter :: FSSFa0 = 169
  integer, parameter :: VFFaSS0 = 170
  integer, parameter :: VSVSVVS0 = 171
  integer, parameter :: VSVSVVS1 = 172
  integer, parameter :: VSVSVVS2 = 173
  integer, parameter :: FSFaSV0 = 174
  integer, parameter :: FVSFaS0 = 175
  integer, parameter :: SFaFS0 = 176
  integer, parameter :: FFaSVS0 = 177
  integer, parameter :: SSVVSVV0 = 178
  integer, parameter :: SSVVSVV1 = 179
  integer, parameter :: SSVVSVV2 = 180
  integer, parameter :: SFFaVS0 = 181
  integer, parameter :: FVFaS0 = 182
  integer, parameter :: VVSVSSV0 = 183
  integer, parameter :: VVSVSSV1 = 184
  integer, parameter :: VVSVSSV2 = 185
  integer, parameter :: SVSVS0 = 186
  integer, parameter :: VVVSVSS0 = 187
  integer, parameter :: VVVSVSS1 = 188
  integer, parameter :: VVVSVSS2 = 189
  integer, parameter :: SVSVSV0 = 190
  integer, parameter :: SFVFaS0 = 191
  integer, parameter :: VSSVS0 = 192
  integer, parameter :: FVFa0 = 193
  integer, parameter :: FaF0 = 194
  integer, parameter :: SSSVVV0 = 195
  integer, parameter :: VFSFaS0 = 196
  integer, parameter :: SFaSVF0 = 197
  integer, parameter :: SVVSS0 = 198
  integer, parameter :: SSVFaF0 = 199
  integer, parameter :: SVVSV0 = 200
  integer, parameter :: SVVSV1 = 201
  integer, parameter :: FFaSS0 = 202
  integer, parameter :: FaSVFS0 = 203
  integer, parameter :: VVVSSSV0 = 204
  integer, parameter :: VVVSSSV1 = 205
  integer, parameter :: VVVSSSV2 = 206
  integer, parameter :: SVSFaF0 = 207
  integer, parameter :: SSSVV0 = 208
  integer, parameter :: VVVVSSS0 = 209
  integer, parameter :: VVVVSSS1 = 210
  integer, parameter :: VVVVSSS2 = 211
  integer, parameter :: FaVFSS0 = 212
  integer, parameter :: SFaSFV0 = 213
  integer, parameter :: SVVSSV0 = 214
  integer, parameter :: VSFaSF0 = 215
  integer, parameter :: VSFaF0 = 216
  integer, parameter :: VSSVSV0 = 217
  integer, parameter :: VFaF0 = 218
  integer, parameter :: SVVS0 = 219
  integer, parameter :: SVVS1 = 220
  integer, parameter :: VVSSV0 = 221
  integer, parameter :: VVSSV1 = 222
  integer, parameter :: VVSSS0 = 223
  integer, parameter :: FFaSSV0 = 224
  integer, parameter :: UUaV0 = 225
  integer, parameter :: SVSFFa0 = 226
  integer, parameter :: FVFaSS0 = 227
  integer, parameter :: VVVVSS0 = 228
  integer, parameter :: FaVFS0 = 229
  integer, parameter :: SSFaF0 = 230
  integer, parameter :: VSSVSVV0 = 231
  integer, parameter :: VSSVSVV1 = 232
  integer, parameter :: VSSVSVV2 = 233
  integer, parameter :: FaVSFS0 = 234
  integer, parameter :: VSVSSV0 = 235
  integer, parameter :: SFSFaV0 = 236
  integer, parameter :: SSFaVF0 = 237
  integer, parameter :: SSVSVVV0 = 238
  integer, parameter :: SSVSVVV1 = 239
  integer, parameter :: SSVSVVV2 = 240
  integer, parameter :: VVVVS0 = 241
  integer, parameter :: FaVF0 = 242
  integer, parameter :: FaSSFV0 = 243
  integer, parameter :: UaVU0 = 244
  integer, parameter :: VSVVVSS0 = 245
  integer, parameter :: VSVVVSS1 = 246
  integer, parameter :: VSVVVSS2 = 247
  integer, parameter :: SVFaSF0 = 248
  integer, parameter :: SSSVVVV0 = 249
  integer, parameter :: SSSVVVV1 = 250
  integer, parameter :: SSSVVVV2 = 251
  integer, parameter :: SVVVSV0 = 252
  integer, parameter :: SVVVSS0 = 253
  integer, parameter :: SFVSFa0 = 254
  integer, parameter :: VFaSSF0 = 255
  integer, parameter :: SVVSVSV0 = 256
  integer, parameter :: SVVSVSV1 = 257
  integer, parameter :: SVVSVSV2 = 258
  integer, parameter :: SSVVVVS0 = 259
  integer, parameter :: SSVVVVS1 = 260
  integer, parameter :: SSVVVVS2 = 261
  integer, parameter :: SSVFFa0 = 262
  integer, parameter :: VV0 = 263
  integer, parameter :: VV1 = 264
  integer, parameter :: FaVSF0 = 265
  integer, parameter :: FaSFVS0 = 266
  integer, parameter :: FaFVS0 = 267
  integer, parameter :: SVVSVS0 = 268
  integer, parameter :: VSVVSSV0 = 269
  integer, parameter :: VSVVSSV1 = 270
  integer, parameter :: VSVVSSV2 = 271
  integer, parameter :: FaFSVS0 = 272
  integer, parameter :: VFSSFa0 = 273
  integer, parameter :: FaSSF0 = 274
  integer, parameter :: VFaFSS0 = 275
  integer, parameter :: VSV0 = 276
  integer, parameter :: VSV1 = 277
  integer, parameter :: VVVSSVS0 = 278
  integer, parameter :: VVVSSVS1 = 279
  integer, parameter :: VVVSSVS2 = 280
  integer, parameter :: SSFFa0 = 281
  integer, parameter :: UVUa0 = 282
  integer, parameter :: VVSSVS0 = 283
  integer, parameter :: VSSV0 = 284
  integer, parameter :: VSSV1 = 285
  integer, parameter :: VVSSVSV0 = 286
  integer, parameter :: VVSSVSV1 = 287
  integer, parameter :: VVSSVSV2 = 288
  integer, parameter :: SVSSVV0 = 289
  integer, parameter :: VSVSSVV0 = 290
  integer, parameter :: VSVSSVV1 = 291
  integer, parameter :: VSVSSVV2 = 292
  integer, parameter :: SFaFSV0 = 293
  integer, parameter :: VUUa0 = 294
  complex(dp), parameter :: c1 = cmplx(sq2/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c1f = "sq2/2"
  complex(dp), parameter :: c2 = cmplx(-sq2/(2d0*Nc), 0d0, kind=dp)
  character(len=20), parameter :: c2f = "-sq2/(2*Nc)"
  complex(dp), parameter :: c3 = cmplx(1d0/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c3f = "1/2"
  complex(dp), parameter :: c4 = cmplx(-1d0/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c4f = "-1/2"
  complex(dp), parameter :: c5 = cmplx(0d0, sq2/2d0, kind=dp)
  character(len=20), parameter :: c5f = "i_*sq2/2"
  complex(dp), parameter :: c6 = cmplx(0d0, -sq2/2d0, kind=dp)
  character(len=20), parameter :: c6f = "-i_*sq2/2"
  complex(dp), parameter :: c7 = cmplx(-1d0/Nc, 0d0, kind=dp)
  character(len=20), parameter :: c7f = "-1/Nc"
  complex(dp), parameter :: c8 = cmplx(1d0, 0d0, kind=dp)
  character(len=20), parameter :: c8f = "1"
  complex(dp), dimension(1:8) :: colorcoeffs = (/c1, c2, c3, c4, c5, c6, c7, c8&
                                               /)
  character(len=20), dimension(1:8) :: colorcoeffs_form = (/c1f, c2f, c3f, c4f &
                 , c5f, c6f, c7f, c8f/)
  integer, dimension(1:294) :: lorentz_rank = (/-1, -1, 0, 0, 0, 0, -1, 0, 0, -&
                                              1, 0, 0, 0, -1, 1, -1, 0, 0, 0, 1&
                                              , -1, -1, 0, 1, -1, 0, 0, 0, 1, 0&
                                              , 0, 0, 1, -1, 0, 0, 0, -1, -1, -&
                                              1, -1, -1, -1, 1, -1, -1, 0, 0, 0&
                                              , 0, -1, -1, -1, 1, 0, 0, 0, 0, 0&
                                              , -1, -1, -1, -1, -1, 1, -1, -1  &
                                              , -1, 0, 0, 0, 0, 0, 0, -1, -1, 0&
                                              , 0, 0, -1, -1, 1, 0, -1, -1, -1 &
                                              , -1, 0, 0, 0, 1, -1, 0, 0, 0, -1&
                                              , 0, 0, 0, 0, 0, 0, 1, -1, -1, 0 &
                                              , 0, 0, -1, -1, 0, 0, 0, 0, -1, -&
                                              1, -1, -1, 1, -1, -1, 0, -1, 1, -&
                                              1, 0, 0, 0, -1, -1, -1, -1, -1, -&
                                              1, -1, 1, 1, 2, -1, -1, -1, 1, -1&
                                              , -1, -1, -1, -1, 0, 0, 0, 0, 1  &
                                              , -1, 1, -1, -1, 1, 1, 1, 0, -1  &
                                              , -1, 1, -1, 1, -1, 0, 1, -1, -1 &
                                              , 0, 0, 0, -1, -1, -1, -1, 0, 0, &
                                              0, -1, -1, 0, 0, 0, 0, 0, 0, 0, 1&
                                              , -1, 1, 1, -1, 1, -1, -1, 0, -1 &
                                              , 1, -1, -1, -1, 0, 0, 0, -1, 1, &
                                              0, 0, 0, -1, -1, 1, -1, -1, 1, 1 &
                                              , 0, -1, 1, -1, 1, -1, 1, -1, -1 &
                                              , 0, -1, -1, 0, 0, 0, -1, 1, -1  &
                                              , -1, 0, 0, 0, 0, 1, -1, 1, 0, 0 &
                                              , 0, -1, 0, 0, 0, 0, 0, -1, -1, 0&
                                              , 0, 0, 0, 0, 0, -1, -1, -1, -1  &
                                              , -1, -1, 0, 0, 0, 0, -1, -1, -1 &
                                              , -1, 2, -1, 0, 0, 0, -1, 1, 1, 2&
                                              , -1, 0, 0, 0, 1, 0, 0, 0, -1, 1 &
                                              /)
  integer, dimension(1:294) :: ff_ids = (/0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                        , 0, 0, 0, 0, 0, 0, 0, 0/)
  integer, parameter :: SVVVS_heft = 156
  integer, parameter :: SVVVV_heft = 114
  integer, parameter :: VVSS_heft = 100
  integer, parameter :: VVSV_heft = 141
  integer, parameter :: VSSV_heft = 102
  integer, parameter :: VVS_heft = 97
  integer, parameter :: VVVSS_heft = 150
  integer, parameter :: SVVV_heft = 143
  integer, parameter :: VVVSV_heft = 111
  integer, parameter :: SVV_heft = 99
  integer, parameter :: SVVSV_heft = 157
  integer, parameter :: VVVSSV_heft = 122
  integer, parameter :: SVVS_heft = 103
  integer, parameter :: VVSSV_heft = 153
  integer, parameter :: VVVVSS_heft = 120
  integer, parameter :: SVVVSV_heft = 131
  integer, parameter :: SVVVVS_heft = 130
  integer, parameter :: VVVVS_heft = 110
  integer, parameter :: VVVS_heft = 140
  integer, parameter :: VSV_heft = 98
  integer, parameter :: SVSV_heft = 104
  integer, dimension(1:294) :: recola_base = (/0, 0, 0, 0, 0, SVVVS_heft, 0, 15&
                                             , SVVVV_heft, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, VVSS_heft, 0, 0, 0,    &
                                             VVSV_heft, 0, 0, 0, 0, 12, 0, 0, 0&
                                             , VVS_heft, 0, 0, 0, 0, 0, 0, 0, 0&
                                             , 0, 0, 162, 0, 0, 0, 0, 0, 0, 0, &
                                             0, 0, 0, 0, 0, 0, 16, -1, 0, 0, 0 &
                                             , 0, 0, SVV_heft, 0, 0, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , VVVSSV_heft, 0, 0, 0, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0,      &
                                             SVVVVS_heft, 0, 0, 0, 0, 0, 0, 0, &
                                             VVVS_heft, 0, 0, -11, 0, SVSV_heft&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 33&
                                             , 0, 164, 0, 0, 0, VVVSS_heft, 0, &
                                             SVVV_heft, 0, VVVSV_heft, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 31, 0, 0, 0, 0, 0, 0, SVVSV_heft&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 53, SVVS_heft &
                                             , 0, VVSSV_heft, 0, 0, 0, 161, 0, &
                                             0, VVVVSS_heft, 0, 0, 0, 0, 0, 0, &
                                             0, 0, 0, 0, 0, 0, VVVVS_heft, 51, &
                                             0, 166, 0, 0, 0, 0, 0, 0, 0,      &
                                             SVVVSV_heft, 0, 0, 0, 0, 0, 0, 0, &
                                             0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, VSV_heft, 0, 0, 0, 0&
                                             , 0, 165, 0, VSSV_heft, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, 0, 163/)
  integer, dimension(1:294) :: propagator_extension = (/0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0/)
  character(len=*), parameter :: model_timestamp = "Mon Aug  3 19:11:01 2020"

  contains
    function get_timestamp_mdl() result(ret_timestamp)
      character(len=len(model_timestamp)) :: ret_timestamp
      ret_timestamp = model_timestamp
    end function get_timestamp_mdl

    function get_lorentz_rank_mdl(l_id) result(ret_lorentz_rank)
      integer, intent(in) :: l_id
      integer :: ret_lorentz_rank
      ret_lorentz_rank = lorentz_rank(l_id)
    end function get_lorentz_rank_mdl

    function get_propagator_extension_mdl(l_id) result(ret_pext)
      integer, intent(in) :: l_id
      integer :: ret_pext
      ret_pext = propagator_extension(l_id)
    end function get_propagator_extension_mdl

    function get_max_leg_multiplicity_mdl() result(ret_max_leg_multiplicity)
      integer :: ret_max_leg_multiplicity
      ret_max_leg_multiplicity = max_leg_multiplicity
    end function get_max_leg_multiplicity_mdl

    function get_fourfermion_id_mdl(l_id) result(ret_ffid)
      integer, intent(in) :: l_id
      integer :: ret_ffid
      ret_ffid = ff_ids(l_id)
    end function get_fourfermion_id_mdl

    function is_model_init_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_init
    end function is_model_init_mdl

    function is_model_filled_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_filled
    end function is_model_filled_mdl

    subroutine createCurrents
      use fill_currents, only: fill_currents_mdl
      implicit none
      call fill_currents_mdl(.false.)
    end subroutine createCurrents

    subroutine check_vertices_mdl
      use fill_currents, only: fill_currents_mdl
      implicit none
      call fill_currents_mdl(.true.)
    end subroutine check_vertices_mdl

    subroutine av(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branch(bid)%color_id = cid
      vertices(vid)%branch(bid)%colorfac_id = cfid
      vertices(vid)%branch(bid)%pext = pext
      vertices(vid)%branch(bid)%lorentz_id = lid
      vertices(vid)%branch(bid)%perm = perm
      vertices(vid)%branch(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branch(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branch(bid)%couplings(i) = cop(i)
      end do
    end subroutine av

    subroutine avCT(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchCT(bid)%color_id = cid
      vertices(vid)%branchCT(bid)%colorfac_id = cfid
      vertices(vid)%branchCT(bid)%pext = pext
      vertices(vid)%branchCT(bid)%lorentz_id = lid
      vertices(vid)%branchCT(bid)%perm = perm
      vertices(vid)%branchCT(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchCT(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchCT(bid)%couplings(i) = cop(i)
      end do
    end subroutine avCT

    subroutine avR2(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchR2(bid)%color_id = cid
      vertices(vid)%branchR2(bid)%colorfac_id = cfid
      vertices(vid)%branchR2(bid)%pext = pext
      vertices(vid)%branchR2(bid)%lorentz_id = lid
      vertices(vid)%branchR2(bid)%perm = perm
      vertices(vid)%branchR2(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchR2(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchR2(bid)%couplings(i) = cop(i)
      end do
    end subroutine avR2

    function get_colourfac_mdl(cfid)
      integer, intent(in) :: cfid
      complex(dp) :: get_colourfac_mdl
      get_colourfac_mdl = colorcoeffs(cfid)
    end function get_colourfac_mdl

    function get_colourfac_form_mdl(cfid)
      integer, intent(in) :: cfid
      character(len=20) :: get_colourfac_form_mdl
      get_colourfac_form_mdl = colorcoeffs_form(cfid)
    end function get_colourfac_form_mdl

  function get_recola_base_mdl(l_id) result(ret_rb)
      integer, intent(in) :: l_id
      integer :: ret_rb
      ret_rb = recola_base(l_id)
    end function get_recola_base_mdl

  subroutine get_vertex_branch_status_mdl(vx,xlp,branch_exists)
    integer, intent(in)  :: vx,xlp
    logical, intent(out) ::branch_exists
    select case (xlp)
    case(TreeBranch:LoopBranch)
      branch_exists = allocated(vertices(vx)%branch)
    case(CTBranch)
      branch_exists = allocated(vertices(vx)%branchCT)
    case(R2Branch)
      branch_exists = allocated(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_branch_status_mdl

  subroutine dealloc_vertex_mdl(vx,xlp)
    integer, intent(in)  :: vx,xlp
    logical ::branch_exists
    if (vx .ge. 0 .and.  vx .lt. n_vertices) then
      select case (xlp)
      case(TreeBranch:LoopBranch)
        if (allocated(vertices(vx)%branch)) deallocate(vertices(vx)%branch)
      case(CTBranch)
        if (allocated(vertices(vx)%branchCT)) deallocate(vertices(vx)%branchCT)
      case(R2Branch)
        if (allocated(vertices(vx)%branchR2)) deallocate(vertices(vx)%branchR2)
      case default
        if (allocated(vertices(vx)%branch)) deallocate(vertices(vx)%branch)
        if (allocated(vertices(vx)%branchCT)) deallocate(vertices(vx)%branchCT)
        if (allocated(vertices(vx)%branchR2)) deallocate(vertices(vx)%branchR2)
      end select
    end if
  end subroutine dealloc_vertex_mdl

  subroutine get_vertex_colour_branches_mdl(vx,xlp,cfids,nbranches)
    integer, allocatable, intent(out) :: cfids(:)
    integer, intent(out) :: nbranches
    integer, intent(in)  :: vx,xlp
    select case (xlp)
    case (TreeBranch:LoopBranch)
      allocate(cfids(size(vertices(vx)%branch)))
      cfids(:) = vertices(vx)%branch(0:)%color_id
      nbranches = size(vertices(vx)%branch)
    case(CTBranch)
      allocate(cfids(size(vertices(vx)%branchCT)))
      cfids(:) = vertices(vx)%branchCT(0:)%color_id
      nbranches = size(vertices(vx)%branchCT)
    case(R2Branch)
      allocate(cfids(size(vertices(vx)%branchR2)))
      cfids(:) = vertices(vx)%branchR2(0:)%color_id
      nbranches = size(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_colour_branches_mdl

  subroutine get_vertex_pext_mdl(vx,xlp,br,pext)
    integer, intent(in) :: vx,xlp,br
    integer, intent(out) :: pext
    select case (xlp)
    case (TreeBranch:LoopBranch)
      pext = vertices(vx)%branch(br)%pext
    case(CTBranch)
      pext = vertices(vx)%branchCT(br)%pext
    case(R2Branch)
      pext = vertices(vx)%branchR2(br)%pext
    end select
  end subroutine get_vertex_pext_mdl

  subroutine get_vertex_order_increase_mdl(vx,xlp,br,orderInc)
    integer, intent(in) :: vx,xlp,br
    integer, dimension(:), intent(out) :: orderInc
    select case (xlp)
    case (TreeBranch:LoopBranch)
      orderInc = vertices(vx)%branch(br)%coupling_order
    case(CTBranch)
      orderInc = vertices(vx)%branchCT(br)%coupling_order
    case(R2Branch)
      orderInc = vertices(vx)%branchR2(br)%coupling_order
    end select
  end subroutine get_vertex_order_increase_mdl

  subroutine get_vertex_lorentz_type_mdl(vx,xlp,br,lid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: lid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      lid = vertices(vx)%branch(br)%lorentz_id
    case(CTBranch)
      lid = vertices(vx)%branchCT(br)%lorentz_id
    case(R2Branch)
      lid = vertices(vx)%branchR2(br)%lorentz_id
    end select
  end subroutine get_vertex_lorentz_type_mdl

  subroutine get_vertex_perm_mdl(vx,xlp,br,perm)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: perm
    select case (xlp)
    case (TreeBranch:LoopBranch)
      perm = vertices(vx)%branch(br)%perm
    case(CTBranch)
      perm = vertices(vx)%branchCT(br)%perm
    case(R2Branch)
      perm = vertices(vx)%branchR2(br)%perm
    end select
  end subroutine get_vertex_perm_mdl

  subroutine get_vertex_colour_id_mdl(vx,xlp,br,cid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      cid = vertices(vx)%branch(br)%colorfac_id
    case(CTBranch)
      cid = vertices(vx)%branchCT(br)%colorfac_id
    case(R2Branch)
      cid = vertices(vx)%branchR2(br)%colorfac_id
    end select
  end subroutine get_vertex_colour_id_mdl

  function is_zero_coupling_vertex_mdl(vx,xlp,br) result(zero)
    use class_couplings, only: get_coupling_value_mdl
    use input_mdl, only: zerocut
    integer, intent(in)  :: vx,xlp,br
    logical              :: zero
    real(dp)             :: maxvalue
    integer              :: i

    zero = .false.
    select case (xlp)
    case (TreeBranch:LoopBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branch(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branch(br)%couplings(i-&
                1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(CTBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchCT(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchCT(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(R2Branch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchR2(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchR2(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    end select
  end function is_zero_coupling_vertex_mdl

  subroutine get_vertex_ncouplings_mdl(vx,xlp,br,ncouplings)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: ncouplings
    select case(xlp)
    case(TreeBranch, LoopBranch)
      ncouplings = size(vertices(vx)%branch(br)%couplings)
    case (CTBranch)
      ncouplings = size(vertices(vx)%branchCT(br)%couplings)
    case (R2Branch)
      ncouplings = size(vertices(vx)%branchR2(br)%couplings)
    end select
  end subroutine get_vertex_ncouplings_mdl

  subroutine get_vertex_coupling_ids_mdl(vx,xlp,br,cids)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cids(:)
    select case(xlp)
    case(TreeBranch, LoopBranch)
      cids(:) = vertices(vx)%branch(br)%couplings(:)
    case (CTBranch)
      cids(:) = vertices(vx)%branchCT(br)%couplings(:)
    case (R2Branch)
      cids(:) = vertices(vx)%branchR2(br)%couplings(:)
    end select
  end subroutine get_vertex_coupling_ids_mdl

  subroutine init_vertices_mdl()
    use input_mdl, only: nx, error_mdl
    use class_particles,only: n_orders,driver_timestamp
    use fill_ctparameters, only: init_CTparameters_mdl
    use fill_couplings,only: fill_couplings_mdl,fill_CTcouplings_mdl

    ! check if modelfile is compatible to drivers
    if (model_timestamp .ne. driver_timestamp) then
      call error_mdl("Model corrupted. Driver timestamp mismatch.", &
                    where='init_vertices')
    end if

    vertices_init = .true.
    call fill_couplings_mdl
    call init_CTparameters_mdl(.true.)
    call fill_CTcouplings_mdl
    call dict_init(1327)
    call createCurrents

  end subroutine init_vertices_mdl

    subroutine get_vertex_lorentz_rank_mdl(ty,cc,mr)
      integer, intent(in) :: ty
      logical, intent(in) :: cc(0:)
      integer, intent(out) :: mr
      select case(ty)
      case(15)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(20)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(24)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(29)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(33)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(44)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(54)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(65)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(82)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(91)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(103)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(119)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(124)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(136)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(137)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(138)
        if (cc(0) .or. cc(1)) then
          mr = 2
        else
          mr = 0
        end if 
      case(142)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(152)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(154)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(157)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(158)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(159)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(163)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(165)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(168)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(190)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(192)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(193)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(195)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(200)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(208)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(214)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(217)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(218)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(221)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(223)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(225)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(235)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(242)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(244)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(276)
        if (cc(0) .or. cc(1)) then
          mr = 2
        else
          mr = 0
        end if 
      case(282)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(283)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(284)
        if (cc(0) .or. cc(1)) then
          mr = 2
        else
          mr = 0
        end if 
      case(289)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(294)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case default
        mr = 0
      end select 
    end subroutine get_vertex_lorentz_rank_mdl
end module class_vertices
