!******************************************************************************!
!                                                                              !
!    lcVSSSVV.f90                                                              !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2020 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcVSSSVV.f90                                   |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2015                                 |            !
!        |       | UFO version: 0.1                               |            !
!        |       | Generation date: 2020-08-03 19:59             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcVSSSVV
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29, x30, x31, x32, x33, x34, x35
  complex (dp), dimension(0:3) :: x0vec, x1vec
  contains
    subroutine loopVSSSVV(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
    , PL4, w5, P5, PL5, m, wp, riMaxIn, riMaxOut, rankInc)
      use class_vertices
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
      complex (kind = dp), intent(in), dimension(0:3) :: w3
      complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
      complex (kind = dp), intent(in), dimension(0:3) :: w4
      complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
      complex (kind = dp), intent(in), dimension(0:3) :: w5
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(VSSSVV0)
!       c0*(Metric(1,5)*P(6,1) - Metric(1,5)*P(6,5) - Metric(1,6)*P(5,1) + Metric
!       (1,6)*P(5,6) + Metric(5,6)*P(1,5) - Metric(5,6)*P(1,6))
        x1 = P2(1) + P3(1) + P4(1)
        x2 = P2(2) + P3(2) + P4(2)
        x3 = P2(3) + P3(3) + P4(3)
        x4 = P2(0) + P3(0) + P4(0)
        x5 = cima*co(0)*w2(0)*w3(0)*w4(0)
        do j = riMaxIn, 0, -1
          x0 = -w5(0)*wl(0,j) + w5(1)*wl(1,j) + w5(2)*wl(2,j) + w5(3)*wl(3,j)
          wp(:,j) = -x5*(P1(:)*x0 - P5(:)*x0 + w5(:)*(-wl(0,j)*(P1(0) + 2*P5(0 &
                    ) + x4) + wl(1,j)*(P1(1) + 2*P5(1) + x1) + wl(2,j)*(P1(2   &
                    ) + 2*P5(2) + x2) + wl(3,j)*(P1(3) + 2*P5(3) + x3)) - wl(:,&
                    j)*(-w5(0)*(2*P1(0) + P5(0) + x4) + w5(1)*(2*P1(1) + P5(1  &
                    ) + x1) + w5(2)*(2*P1(2) + P5(2) + x2) + w5(3)*(2*P1(3) +  &
                    P5(3) + x3)))
          if(rankInc .lt. 1) cycle
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + x5*(-gg(:,mu)*x0 + w5(:)*wl(mu,j) - 2* &
                          w5(mu)*wl(:,j))
          end do 
        end do 
      end select 
    end subroutine loopVSSSVV
end module lcVSSSVV
