!******************************************************************************!
!                                                                              !
!    tcVSSV.f90                                                                !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2020 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   tcVSSV.f90                                     |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2015                                 |            !
!        |       | UFO version: 0.1                               |            !
!        |       | Generation date: 2020-08-03 19:59             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module tcVSSV
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29, x30, x31, x32, x33, x34, x35, x36, x37,  &
                  x38, x39, x40, x41, x42, x43, x44, x45, x46, x47, x48, x49,  &
                  x50, x51
  complex (dp), dimension(0:3) :: x0vec, x1vec, x2vec, x3vec, x4vec, x5vec
  contains
    subroutine treeVSSV(ty, co, w1, P1, PL1, w2, P2, PL2, w3, P3, PL3, den, m, &
    wout, last, VB_last)
      use class_vertices
      integer, intent(in) :: ty
      logical, intent(inout) :: VB_last
      logical, intent(in) :: last
      complex(kind=dp), dimension(0:), intent(in) :: co
      complex(kind=dp), intent(in), dimension(0:3) :: P1, PL1
      complex(kind=dp), intent(in), dimension(0:3) :: w1
      complex(kind=dp), intent(in), dimension(0:3) :: P2, PL2
      complex(kind=dp), intent(in), dimension(0:3) :: w2
      complex(kind=dp), intent(in), dimension(0:3) :: P3, PL3
      complex(kind=dp), intent(in), dimension(0:3) :: w3
      complex(kind=dp), intent(in) :: m
      complex(kind=dp), intent(in) :: den
      complex(kind=dp), dimension(0:3), intent(out) :: wout
      character(len=3) :: ty_str
      complex(kind=dp), dimension(0:3) :: pl, p
      select case(ty)
      case(VSSV0)
!       P(1,4)*P(4,1)*c0
!       Metric(1,4)*P(-1,1)*P(-1,4)*c1
        if (last) VB_last = .true.
        x0 = P2(1) + P3(1)
        x1 = P2(2) + P3(2)
        x2 = P2(3) + P3(3)
        x3 = P2(0) + P3(0)
        x0vec = co(1)*w1(:)
        wout = -w2(0)*w3(0)*(-P1(0)**2*x0vec + P1(:)*co(0)*(-w1(0)*(P1(0) + x3 &
               ) + w1(1)*(P1(1) + x0) + w1(2)*(P1(2) + x1) + w1(3)*(P1(3) + x2 &
               )) + x0vec*(P1(1)**2 + P1(2)**2 + P1(3)**2) + x0vec*(-P1(0)*x3  &
                + P1(1)*x0 + P1(2)*x1 + P1(3)*x2))
      case(VSSV1)
!       c0*(P(1,1)*P(4,1) + P(1,4)*P(4,4))
!       P(1,1)*P(4,4)*c1
!       P(1,4)*P(4,1)*c2
!       c3*(Metric(1,4)*P(-1,1)**2 + Metric(1,4)*P(-1,4)**2)
!       Metric(1,4)*P(-1,1)*P(-1,4)*c4
        if (last) VB_last = .true.
        x0 = P2(1) + P3(1)
        x1 = P2(2) + P3(2)
        x2 = P2(3) + P3(3)
        x3 = P2(0) + P3(0)
        x4 = -w1(0)*x3
        x0vec = P2(:) + P3(:)
        x5 = w1(1)*(P1(1) + x0) + w1(2)*(P1(2) + x1) + w1(3)*(P1(3) + x2)
        x6 = P1(0)*w1(0)
        x7 = 2*co(3)
        x8 = -co(4) + x7
        x9 = P1(2)**2
        x10 = P1(3)**2
        x11 = -P1(0)*x3 + P1(1)*x0
        wout = -w2(0)*w3(0)*(P1(:)*co(2)*(-w1(0)*(P1(0) + x3) + x5) - co(0)*(P1&
               (:)*(w1(1)*(2*P1(1) + x0) + w1(2)*(2*P1(2) + x1) + w1(3)*(2*P1(3&
               ) + x2) + x4) + x0vec*(x4 + x5) - x6*(2*P1(:) + x0vec)) + co(1  &
               )*(P1(:) + x0vec)*(P1(1)*w1(1) + P1(2)*w1(2) + P1(3)*w1(3) - x6 &
               ) + co(4)*w1(:)*(P1(2)*x1 + P1(3)*x2 + x11) - w1(:)*x7*(P1(2)*P2&
               (2) + P1(3)*P2(3) - P2(0)*P3(0) + P2(1)*P3(1) + P3(2)*(P1(2) +  &
               P2(2)) + P3(3)*(P1(3) + P2(3)) + x11) - w1(:)*(-P1(0)**2*x8 + P1&
               (1)**2*x8 - P2(0)**2*co(3) + P2(1)**2*co(3) + P2(2)**2*co(3) +  &
               P2(3)**2*co(3) - P3(0)**2*co(3) + P3(1)**2*co(3) + P3(2)**2*co(3&
               ) + P3(3)**2*co(3) - co(4)*x10 - co(4)*x9 + x10*x7 + x7*x9))
      end select 
    end subroutine treeVSSV
end module tcVSSV
